#!/usr/bin/env python3

'''Parse binary format Baidu Browser Android uses to check for updates.'''

import sys
import zlib

def read(buf, n):
    result = buf[:n]
    if len(result) < n:
        raise RuntimeError('payload too small')
    return result

def parse_leaves(mv):
    lst = []
    while mv:
        type = int.from_bytes(read(mv, 2), 'little')
        mv = mv[2:]
        if type < 32767:
            if type < 8191:
                intwidth = 1
            elif type < 16383:
                intwidth = 2
            else:
                intwidth = 4
            n = int.from_bytes(read(mv, intwidth), 'little')
            mv = mv[intwidth:]
            lst.append((type, n))
        else:
            strlen = int.from_bytes(read(mv, 2), 'little')
            mv = mv[2:]
            string = bytes(read(mv, strlen)).decode('utf-8')
            mv = mv[strlen:]
            lst.append((type, string))
    return lst

def parse_branch(mv):
    lst = []
    while mv:
        type = int.from_bytes(read(mv, 2), 'little')
        leaves_length = int.from_bytes(read(mv[2:], 2), 'little')
        leaves_end = 8 + leaves_length
        branches_length = int.from_bytes(read(mv[4:], 4), 'little')
        branches_end = leaves_end + branches_length
        if branches_end > len(mv):
            raise RuntimeError('payload size too large: %d > %d' % (branches_end, len(mv)))
        leaves = parse_leaves(mv[8:leaves_end])
        if type < 61439 and branches_length > 0:
            branches = read(mv[leaves_end:branches_end], branches_length).decode('utf-16le')
        elif type < 65535:
            branches = parse_branch(mv[leaves_end:leaves_end + branches_length])
        lst.append((type, leaves, branches))
        mv = mv[branches_end:]
    return lst

def parse_body(mv):
    length = len(mv)
    magic = read(mv, 4)
    mv = mv[4:]
    if magic != b'db1\x00':
        raise RuntimeError('wrong magic number')
    reported_length = int.from_bytes(read(mv, 4), 'little')
    mv = mv[4:]
    if reported_length != length:
        raise RuntimeError('length mismatch')
    n = int.from_bytes(read(mv, 2), 'little')
    mv = mv[2:]
    entries = []
    for _ in range(n):
        x = int.from_bytes(read(mv, 4), 'little')
        entry = x >> 28, x & 0xfffffff
        entries.append(entry)
        mv = mv[4:]
    data = []
    for i in range(n):
        type, start = entries[i]
        if i < n - 1:
            _, end = entries[i + 1]
        else:
            end = length
        entry_size = end - start
        if entry_size > 0:
            entry_mv = mv[:entry_size]
            if type == 0 or type == 1:
                uncompressed_length = int.from_bytes(read(mv, 4), 'little')
                entry_mv = entry_mv[4:]
                decompressed = zlib.decompress(entry_mv)
                if uncompressed_length != len(decompressed):
                    raise RuntimeError('decompressed length mismatch')
                branch = parse_branch(decompressed)
            else:
                branch = parse_branch(entry_mv)
            data.append((type, branch))
            mv = entry[entry_size:]
    return data

def main():
    buf = sys.stdin.buffer.read()
    print(repr(parse_body(memoryview(buf))))

if __name__ == '__main__':
    main()
